/* id3dir.c

   ID3Dir utility for DOS.

   Works like dir, but displays id3 information alongside each
   filename.

   By Matt Craven.
   (c)2000 Hedgehog Software

   http://www.damp-mp3.co.uk/

   This tool was suggested by Burnfire.
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <conio.h>
#include "id3.h"

struct text_info id3dir_text_info;     /* So we can grab the size of the
                                          display */

/* options variables */

int id3dir_opt_paginate = 0;           /* pause after each page? */
int id3dir_opt_trackno  = 0;           /* display track numbers? */

/*===================================================================
  id3dir_get_filename(filename)

  Given a full filename with path, eg:
      c:\stuff\things\file.ext
  this returns a pointer to the filename part, ie "file.ext"
  =================================================================*/

char *id3dir_get_filename(char *filename)
{
   int n;

   /* start at the end :-) */

   n = strlen(filename)-1;

   /* Look backwards for the "\" */

   while(n > 0)
      if(filename[n] == '\\')
      {
         n++;
         break;
      }
      else
         n--;

   /* return the pointer */

   return &filename[n];
}

/*===================================================================
  id3dir_clamp_printf(txt)

  This outputs the text in "txt" to the screen, but stops if it
  reaches the end of the line
  =================================================================*/

void id3dir_clamp_printf(char *txt)
{
   int x = wherex();
   int n = 0;

   while(n<strlen(txt) && x < id3dir_text_info.screenwidth)
   {
      printf("%c",txt[n]);
      n++;
      x++;
   }
}

/*===================================================================
  id3dir_trim(txt)

  ID3 tag fields are generally padded with spaces.  This function
  inserts the string terminator (0) at the actual end of the text
  in the given "txt" string.
  =================================================================*/

void id3dir_trim(char *txt)
{
   /* start at the end */

   int n = strlen(txt);

   /* scan back until we hit a printable character */

   while(n > 0 && txt[n] <= 32)
      n--;

   /* insert the string terminator */

   txt[n+1] = 0;
}

/*===================================================================
  main()

  Runs the show...
  =================================================================*/

int main(int argc, char **argv)
{
   int n,t;                /* loop counters */
   FILE *f;                /* for checking the file exists */
   long num_lines_printed; /* remember no. of lines printed */
   ID3_TAG id3;            /* for storing the ID3 tag */
   int maxlen;             /* maximum filename length */
   char temp_txt[16];      /* Temporary text string */

   /* Check there's at least one argument */

   if(argc < 2)
   {
      /* no arguments, so re-invoke with *.* */

      system("id3dir.exe *.*");
      return(0);
   }

   /* Determine the size of the terminal we're printing to (so we
      can clamp output and pause after a screen of text */

   gettextinfo(&id3dir_text_info);

   /* Check for options, and calculate maximum filename length */

   maxlen = 0;
   for(n = 1; n < argc; n++)
   {
      if(stricmp(argv[n], "/H")==0)
      {
         printf("\nUsage: id3dir {options} <filename(s)>\n\n");
         printf("  where {options} is none, one, or more of:\n\n");
         printf("  /H    :  Display this help\n");
         printf("  /P    :  Stop after each page\n");
         printf("  /T    :  Also display track number\n");
         printf("\nOptions should be separated by spaces.\n\n");
         return(0);
      }

      if(stricmp(argv[n], "/P")==0)
         id3dir_opt_paginate = 1;
      if(stricmp(argv[n], "/T")==0)
         id3dir_opt_trackno = 1;

      if(strlen(id3dir_get_filename(argv[n])) > maxlen)
         maxlen = strlen(id3dir_get_filename(argv[n]));
   }

   /* Process specified files */

   num_lines_printed = 0;  /* because we haven't output anything yet */

   for(n = 1; n < argc; n++)
   {

      /* only process this argument if it wasn't an option */

      if(argv[n][0] != '/')
      {

         /* check the file exists */

         f=fopen(argv[n],"rb");

         if(f != NULL)
         {
            /* the file does exist, so process it */

            fclose(f);

            /* output the filename */

            printf(id3dir_get_filename(argv[n]));

            /* if it's less than "max" chars, pad with spaces */

            for(t=strlen(id3dir_get_filename(argv[n])); t<maxlen+2; t++)
            {
               printf(" ");
            }

            /* If it has a tag, output it... */

            if(id3_read(argv[n], &id3))
            {
               id3dir_trim(id3.artist);
               id3dir_trim(id3.title);

               id3dir_clamp_printf(id3.artist);
               id3dir_clamp_printf(" - ");

               if(id3dir_opt_trackno)
               {
                  /* display track number */

                  sprintf(temp_txt,"%d",id3.track);
                  id3dir_clamp_printf(temp_txt);
                  id3dir_clamp_printf(" - ");
               }

               id3dir_clamp_printf(id3.title);
            }

            /* end the line, and remember how many lines we've printed */

            printf("\n");
            num_lines_printed++;

            /* pause if the user wanted us to, and we've reached the
               bottom of the page */

            if(id3dir_opt_paginate
            && num_lines_printed > id3dir_text_info.screenheight-4)
            {
               printf("Press a key to continue . . .\n\n");
               getch();
               num_lines_printed = 0;
            }
         }
      }
   }

   /* ok, we're done... */

   return 0;
}
